@@ -27,6 +27,8 @@ import copy
 import json
 import keyring
 
+from atbu.common.mp_global import get_verbosity_level
+
 from ..common.constants import *
 from ..common.exception import *
 from ..common.util_helpers import (
@@ -141,20 +143,17 @@ def add_path_value(
         value_cfg = value_cfg[s]
     value_cfg[section_value_path[-1]] = value
 
+
 def prompt_user_password_file_does_not_exist(default_filename: str):
     is_exist = default_filename is not None and os.path.isfile(default_filename)
-    print(
-        f"Specify a path to an existing password .json file."
-    )
+    print(f"Specify a path to an existing password .json file.")
     def_qual = ""
     if is_exist:
         def_qual = "<ENTER> for the default, "
         print(
             f"If you do not specify a name and press <ENTER>, the default name will be used."
         )
-        print(
-            f"Default path (file exists): {default_filename}"
-        )
+        print(f"Default path (file exists): {default_filename}")
     a = input(f"Enter the path, {def_qual}or 'n' to abort:")
     if a.lower() in ["n", "no"]:
         return None
@@ -162,6 +161,7 @@ def prompt_user_password_file_does_not_exist(default_filename: str):
         return default_filename
     return a
 
+
 def restore_keyring_secrets(storage_def: dict):
     keyring_section: dict = storage_def.get(CONFIG_SECTION_KEYRING_MAPPING)
     if not keyring_section:
@@ -186,10 +186,10 @@ def restore_keyring_secrets(storage_def: dict):
         if password_type == CONFIG_PASSWORD_TYPE_FILENAME:
             filename = base64.b64decode(cba_password.decode("utf-8")).decode("utf-8")
             while not os.path.isfile(filename):
-                print(
-                    f"The credential file does not exist: {filename}"
+                print(f"The credential file does not exist: {filename}")
+                user_specified_filename = prompt_user_password_file_does_not_exist(
+                    filename
                 )
-                user_specified_filename = prompt_user_password_file_does_not_exist(filename)
                 if user_specified_filename is None:
                     raise CredentialSecretFileNotFoundError(
                         f"An existing credential password filename was not specified, aborting restore."
@@ -591,8 +591,13 @@ configuration, you can choose to have one created for you.
         )
         return storage_def
 
-    def get_storage_def_dict(self, storage_def_name) -> dict:
-        return self.get_storage_defs_section().get(storage_def_name)
+    def get_storage_def_dict(self, storage_def_name, must_exist: bool = False) -> dict:
+        storage_def_dict = self.get_storage_defs_section().get(storage_def_name)
+        if must_exist and storage_def_dict is None:
+            raise StorageDefinitionNotFoundError(
+                f"The storage definition '{storage_def_name}' not found."
+            )
+        return storage_def_dict
 
     def get_only_storage_def_dict(self) -> dict:
         storage_def_section: dict = self.get_storage_defs_section()
@@ -604,6 +609,31 @@ configuration, you can choose to have one created for you.
     def is_storage_def_exists(self, storage_def_name) -> bool:
         return self.get_storage_def_dict(storage_def_name=storage_def_name) is not None
 
+    def get_storage_def_compression_section(self, storage_def_name) -> dict:
+        storage_def_dict = self.get_storage_def_dict(
+            storage_def_name=storage_def_name,
+            must_exist=True,
+        )
+        return storage_def_dict.get(CONFIG_SECTION_COMPRESSION)
+
+    def get_compression_settings_deep_copy(self, storage_def_name) -> dict:
+        compression_sect = self.get_storage_def_compression_section(
+            storage_def_name=storage_def_name,
+        )
+        if compression_sect is not None:
+            compression_sect = copy.deepcopy(compression_sect)
+        else:
+            compression_sect = {}
+        if not isinstance(compression_sect, dict):
+            raise ConfigSectionNotFound(
+                f"For '{storage_def_name}' compression section, "
+                f"expected dict but got {type(compression_sect)}"
+            )
+        for k, v in ATBU_BACKUP_COMPRESSION_DEFAULTS.items():
+            if k not in compression_sect:
+                compression_sect[k] = v
+        return compression_sect
+
     def rename_storage_def(self, old_name, new_name):
         """Rename an existing storage def in this configuration."""
         AtbuConfig.rename_section_storage_def(
