@@ -13,15 +13,16 @@
 # limitations under the License.
 r"""Main entry point, argument parsing.
 """
-
+# pylint: disable=missing-class-docstring
+# pylint: disable=too-many-lines
 import argparse
 import logging
+import multiprocessing
 from typing import Union
 
 from .constants import *
 from .exception import (
     AtbuException,
-    GlobalContextAlreadySet,
     InvalidCommandLineArgument,
     InvalidStateError,
     QueueListenerNotStarted,
@@ -30,12 +31,17 @@ from .exception import (
 )
 from .mp_global import (
     deinitialize_logging,
+    get_verbosity_level,
     global_init,
     initialize_logging_basic,
     initialize_logging,
     remove_created_logging_handlers,
     remove_root_stream_handlers,
 )
+from ..backup.backup_core import (
+    BACKUP_COMPRESSION_CHOICES,
+    BACKUP_COMPRESSION_DEFAULT,
+)
 from ..backup.backup_cmdline import handle_backup
 from ..backup.restore import handle_restore, handle_decrypt
 from ..backup.verify import handle_verify
@@ -126,12 +132,12 @@ def create_argparse():
     # )
 
     # Uncomment to allow --debug-server (for use with VS Code pydebug)
-    # parser.add_argument(
-    #     "--debug-server",
-    #     help=argparse.SUPPRESS, #"Activate the debug server to listen on specified port, wait for a client connect."
-    #     type=int,
-    #     required=False,
-    # )
+    parser.add_argument(
+        "--debug-server",
+        help=argparse.SUPPRESS,  # "Activate the debug server to listen on specified port, wait for a client connect."
+        type=int,
+        required=False,
+    )
 
     #
     # Common to all parser
@@ -304,13 +310,20 @@ the file will not be included in the backup.
         "--dbr",
         "--detect-sneaky-corruption",
         "--dsc",
+        action=argparse.BooleanOptionalAction,
+        default=True,
         help=f"""Enabled by default, reports potential bitrot (aka "sneaky corruption") as an error.
 Requires use of --incremental-plus (--ip). Use --no-detect-bitrot (--no-dbr) to
 squelch reporting as an error (still reported informationally). Bitrot or so-called
 sneaky corruption is detected when a file's date/time and size have remained the
 same since the last backup, but the digests are different.""",
-        action=argparse.BooleanOptionalAction,
-        default=True,
+    )
+    parser_backup.add_argument(
+        "-z",
+        "--compression",
+        choices=BACKUP_COMPRESSION_CHOICES,
+        help=f"""Set the backup compression level. The default is '{BACKUP_COMPRESSION_DEFAULT}'.
+""",
     )
     parser_backup.set_defaults(func=handle_backup)
 
@@ -584,7 +597,8 @@ fail to properly backup your locally stored backup information (which includes t
         help="Delete without a confirmation prompt.",
     )
     cred_delete_storage_def_parser.add_argument(
-        "--delete-backup-info", "--dbi",
+        "--delete-backup-info",
+        "--dbi",
         action=argparse.BooleanOptionalAction,
         help="""If you specify --no-delete-backup-info (--no-dbi), backup information files
 will not be deleted. By default, a storage definition's backup informatiion
@@ -641,16 +655,6 @@ recommended you backup configurations before overwriting them by using
 """,
     )
 
-    #
-    # creds set-username:
-    #
-    # TODO:
-    # creds_set_username_parser = subparser_creds.add_parser(
-    #    "set-username",
-    #    help="Set the user name (aka 'key')associated the storage secret for specified storage definition.",
-    #    parents=[parser_storage_def_specifier, parser_key_type],
-    # )
-
     #
     # creds set-password:
     #
@@ -888,7 +892,7 @@ creates a single database at the root of the specified location.
         help=f"""One or more local file system directories whose persistent information to update,
 each optionally prefixed with the desired file info persistence to use (default is per-dir).
 Note, there is a space after per-file:, per-dir:, and any location. For example,
-'per-file: c:\SomeLocationThatShouldUsePerFile'.
+'per-file: c:\\SomeLocationThatShouldUsePerFile'.
 """,
     )
     parser_update_digests.set_defaults(func=handle_update_digests)
@@ -915,7 +919,7 @@ of removing or moving discovered duplicates.
         help=f"""You must specify this twice, once each for location A and B, optionally prefixing
 either with the desired file info persistence to use (default is per-dir).
 Note, there is a space after per-file:, per-dir:, and any location. For example,
-'per-file: c:\SomeLocationThatIsUsingPerFile'.
+'per-file: c:\\SomeLocationThatIsUsingPerFile'.
 """,
     )
     parser_diff.add_argument(
@@ -1054,10 +1058,8 @@ def wait_for_debugger_attach(debug_server_port: Union[str, int, list]):
 
 
 def main(argv=None):
-    try:
-        global_init()
-    except GlobalContextAlreadySet:
-        pass
+    multiprocessing.set_start_method("spawn")
+    global_init()
     initialize_logging_basic()
     # pdb.set_trace()
     logging.info(f"{ATBU_ACRONYM} - v{ATBU_VERSION_STRING}")
@@ -1077,16 +1079,15 @@ def main(argv=None):
             f"Expected help subject to match those available: {args.help_subject}"
         )
 
+    verbosity_level = 0
     if hasattr(args, "verbosity") and args.verbosity is not None:
-        set_verbosity_level(args.verbosity)
+        verbosity_level = args.verbosity
 
     # Instantiate global singleton GlobalHasherDefinitions.
     try:
         GlobalHasherDefinitions([DEFAULT_HASH_ALGORITHM])
     except SingletonAlreadyCreated:
-        logging.warning(
-            f"GlobalHasherDefinitions already created."
-        )  # TODO: determine proper pytest cleanup and/or set is_test variable etc.
+        logging.warning(f"GlobalHasherDefinitions already created.")
 
     logfile = None
     loglevel = None
@@ -1113,7 +1114,12 @@ def main(argv=None):
         if hasattr(args, "func"):
             remove_created_logging_handlers()
             remove_root_stream_handlers()
-            initialize_logging(logfile, loglevel, log_console_detail)
+            initialize_logging(
+                logfile=logfile,
+                loglevel=loglevel,
+                verbosity_level=verbosity_level,
+                log_console_detail=log_console_detail,
+            )
             exit_code = args.func(args)
             if exit_code is None:
                 exit_code = 0
