@@ -13,6 +13,9 @@
 # limitations under the License.
 r"""ATBU Exceptions.
 """
+
+# pylint: disable=missing-class-docstring
+
 from dataclasses import dataclass
 import sys
 import traceback
@@ -23,12 +26,14 @@ ANOMALY_KIND_EXCEPTION = "exception"
 ANOMALY_KIND_CANCELLED = "cancelled"
 ANOMALY_KIND_UNEXPECTED_STATE = "unexpected state"
 
+
 @dataclass
 class Anomaly:
     kind: ANOMALY_KINDS
     exception: Exception = None
     message: str = None
 
+
 class AtbuException(Exception):
     def __init__(self, message: str, cause=None):
         super().__init__(message)
@@ -147,12 +152,6 @@ class RetryLimitReached(AtbuException):
         super().__init__(message=message, cause=cause)
 
 
-class GlobalContextAlreadySet(AtbuException):
-    def __init__(self, message: str = None, cause=None):
-        self._cause = cause
-        super().__init__(message=message, cause=cause)
-
-
 class GlobalContextNotSet(AtbuException):
     def __init__(self, message: str = None, cause=None):
         self._cause = cause
@@ -387,6 +386,12 @@ class PreambleParsingError(BackupException):
         super().__init__(message=message, cause=cause)
 
 
+class StateNotYetKnownError(BackupException):
+    def __init__(self, message: str = None, cause=None):
+        self._cause = cause
+        super().__init__(message=message, cause=cause)
+
+
 class InvalidCommandLineArgument(AtbuException):
     def __init__(self, message: str = None, cause=None):
         self._cause = cause
@@ -410,11 +415,37 @@ class InvalidBase64StringError(AtbuException):
         self._cause = cause
         super().__init__(message=message, cause=cause)
 
+
 class PipelineResultIsNotPipelineWorkItem(AtbuException):
     def __init__(self, message: str = None, cause=None):
         self._cause = cause
         super().__init__(message=message, cause=cause)
 
+
+class PipelineLastStageError(AtbuException):
+    def __init__(self, message: str = None, cause=None):
+        self._cause = cause
+        super().__init__(message=message, cause=cause)
+
+
+class PipelineFutureStuckPendingError(AtbuException):
+    def __init__(self, message: str = None, cause=None):
+        self._cause = cause
+        super().__init__(message=message, cause=cause)
+
+
+class InvalidPipeConnectionMessage(AtbuException):
+    def __init__(self, message: str = None, cause=None):
+        self._cause = cause
+        super().__init__(message=message, cause=cause)
+
+
+class PipeConnectionAlreadyEof(AtbuException):
+    def __init__(self, message: str = None, cause=None):
+        self._cause = cause
+        super().__init__(message=message, cause=cause)
+
+
 def exc_to_string(ex: Exception):
     return f"ex={ex} details: {traceback.format_exception(ex, ex, ex.__traceback__)}"
 
